#!/usr/bin/python3
#  coding: utf-8
#
#       mappingPressToSolidFace.py
#
#       流体側の圧力からFrontISTRを実行して変位を出力する。
#
#   23/09/22    新規作成
#

import sys, os
import time
import pickle
import pyCoupling
import shutil
import mmap
import multiprocessing
import mappingFromPointCloud as mapping
import pyFistr_ptc as pyFistr

waitCount = 5000
waitTime = 0.01

#--------------------
#  createCommonArea
#--------------------
def createCommonArea():
    """ common fileを共有メモリとして設定する。
    200文字で設定。"""
    fluidCaseDir = couplingDict["fluidCaseDir"]
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    name = "common"
    commonFile = dataDir + "/" + name
    f = open(commonFile, "r+b"); mmFile = mmap.mmap(f.fileno(), 0); f.close()
    return mmFile

#
#  writeCommonArea
def writeCommonArea(cont):
    """ 共有メモリ（200文字）への書き込み。"""
    global mmCommon
    length = 200
    space = "\00" * (length - len(cont))
    cont += space
    mmCommon.seek(0)
    mmCommon.write(cont.encode())
    mmCommon.flush()

#
#  readCommonArea
def readCommonArea():
    """ 共有メモリの内容を読み込む"""
    global mmCommon
    mmCommon.seek(0)
    cont = mmCommon.read().decode()
    n = cont.find("\00")
    cont = cont[:n]
    return cont

#----------------------
#  coupling_fsi
#----------------------
def coupling_fsi():
    """ 圧力dataをsolidFaceにmappingしてFrontISTRを実行、変位を計算する。"""
    patchName = couplingDict["mappingPatch"]
    timeName = couplingDict["timeName"]
    procNo = couplingDict["procNo"]
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    #--- 圧力をmapping ------
    #ptcDataの読み込み
    pressFile = dataDir + "/" + patchName + "_facePress_" + timeName + "_" + procNo + "tm"
    #圧力の点群ファイルを作成
    createPtcFile(pressFile)
    if int(procNo) == 0:
        #圧力をmappingしてcntファイルに反映
        createMappingPress()
        #--- fistr実行 ----------
        #fsi計算前の設定
        endStep, endTime = setFrontIstr_fsi()
        #Fistr起動
        runFistr("fsi", endStep, endTime)
        #--- 変位を取得 ---------
        #変位のptcデータを作成
        print("(py) --- creating displacement PTC ...", flush=True)
        dispPtc = createDispPtc(endStep)
        #変位のmapping結果fileを作成
        print("(py) --- displacement value is mapping to patch '" + patchName + "' ...", flush=True)
        createMappingDispFile(dispPtc, endStep)
        #--- 結果file整理（file削除、restart設定）-----
        deleteResSetRestartFileFsi()
    return ""

#
#  waitUntilExistingFile
#-------------------------
def waitUntilExistingFile(fileName):
    """ pressFileが出来上がるまで待つ"""
    count = waitCount
    sleepTime = waitTime
    while True:
        if os.path.exists(fileName) == True:
            time.sleep(sleepTime)
            break
        count -= 1
        if count < 0:
            break
        time.sleep(sleepTime)
    return

#
#  createPtcFile
#----------------
def createPtcFile(pressFile):
    """ 圧力の点群file作成"""
    #faceCenterLoc読み込み
    patchName = couplingDict["mappingPatch"]
    timeName = couplingDict["timeName"]
    solidCaseDir = couplingDict["solidCaseDir"]
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data"
    faceCenterFile = dataDir + "/" + patchName + "_faceCenterLocs_" + procNo
    f = open(faceCenterFile); lines = f.readlines(); f.close()
    #facePressを読み込み
    fp = open(pressFile); pLines = fp.readlines(); fp.close()
    newLines = []
    for i in range(len(lines)):
        newLine = lines[i][:-1] + " " + pLines[i]
        newLines.append(newLine)
    #圧力のptcファイルを保存
    name = patchName + "_faceCenterPress_" + timeName + "_" + procNo + ".particles"
    ptcFileName = solidCaseDir + "/" + name
    f = open(ptcFileName, "w"); f.writelines(newLines); f.close()
    return

#
#  createMappingPress
#---------------------
def createMappingPress():
    """ 各procNo毎の圧力点群fileをFistr側にmappingする"""
    #各procno毎の点群fileをまとめる
    procNo = couplingDict["procNo"]
    nProcs = couplingDict["nProcs"]
    patchName = couplingDict["mappingPatch"]
    timeName = couplingDict["timeName"]
    sgrpName = couplingDict["mappingSgrp"]
    solidCaseDir = couplingDict["solidCaseDir"]
    sgrpDictName = solidCaseDir + "/sgrpDict.bin"
    f = open(sgrpDictName, "rb")
    sgrpDict = pickle.load(f)
    f.close()
    nodeDictName = solidCaseDir + "/nodeDict.bin"
    f = open(nodeDictName, "rb")
    nodeDict = pickle.load(f)
    f.close()
    #各procno毎の点群fileをまとめる
    lines = []
    if int(procNo) == 0:
        for n in range(int(nProcs)):
            pNo = str(n)
            name = patchName + "_faceCenterPress_" + timeName + "_" + pNo + ".particles"
            eachPtcFile = solidCaseDir + "/" + name
            waitUntilExistingFile(eachPtcFile)
            f = open(eachPtcFile)
            lines += f.readlines()
            f.close()
        newName = patchName + "_faceCenterPress.particles"
        allPtcFile = solidCaseDir + "/" + newName
        f = open(allPtcFile, "w"); f.writelines(lines); f.close()
        #ptcData(数値data）を取得
        mesh = mapping.fistrMesh(solidCaseDir)
        ptcData = mesh.getPtcDataFromLines(lines)
        #mapping用の3点を取得
        mapStat, mappingTriDict = mesh.getMappingTriLocs(ptcData, sgrpDict, nodeDict)
        #圧力をmapping
        print("(py) --- pressure value is mapping to SGRP '" + sgrpName + "' of FrontISTR...", flush=True)
        mappingDict = mesh.setMappingDataFromTriLocs(ptcData, sgrpDict, mappingTriDict)
        #mapping結果を保存
        lines = mesh.createMappingFile(mappingDict, sgrpName, allPtcFile)
        name = "mappingSP_" + sgrpName + ".map"
        fileName = solidCaseDir + os.sep + name
        f = open(fileName, "w"); f.writelines(lines); f.close()
        #mapping結果をcntファイルに反映
        remakeFistrCntFileForPress(lines)

#
#  remakeFistrCntFileForPress
def remakeFistrCntFileForPress(mapLines):
    """ 圧力のmapping結果をcntファイルに反映"""
    mapCoeff = couplingDict["mappingCoeff"]
    mapShift = couplingDict["mappingShift"]
    timeName = couplingDict["timeName"]
    solidCaseDir = couplingDict["solidCaseDir"]
    sgrpName = couplingDict["mappingSgrp"]
    #追加行を作成
    coeff = float(mapCoeff)
    shift = float(mapShift)
    addLines = []
    for line in mapLines:
        if line[0] != "#":
            words = line.split()
            if len(words) >= 3:
                #words[2] = str(float(words[2]) * coeff + shift)
                press = float(words[2]) * coeff + shift
                #計算初回のみ値を修正
                #if iniFlag == 0
                if float(timeName) == 0.0:
                    #値が「0」の場合、zeroに近い値に修正
                    if press == 0.0:
                        press = 1.0e-36     #zeroに近い値
                words[2] = str(press)
                words[1] = "P" + words[1]
                newLine = ", ".join(words) + "\n"
                addLines.append(newLine)
    #cntFileを修正
    cntFile = solidCaseDir + os.sep + "FistrModel_fsi.cnt"
    f = open(cntFile); lines = f.readlines(); f.close()
    #挿入場所は、最初の!DLOADの該当SGRPに挿入する。
    i = 0
    while i < len(lines):
        i = pyFistr.getNextHeaderName(lines, i, "!DLOAD", "")
        i += 1
        if i >= len(lines):
            mess = "error: '" + sgrpName + "' does not exist in !DLOAD.\n"
            print(mess)
        words = pyFistr.deleteSp(lines[i]).split(",")
        if words[0] == sgrpName:
            si = i - 1
            ei = pyFistr.skipNextHeader(lines, i)
            break
    if si < len(lines):
        #cntFile修正、書き込み
        name = "mappingSP_" + sgrpName + ".map"
        grpid = pyFistr.getValName(lines[si], "GRPID")
        words = ["!DLOAD", "GRPID="+grpid, "forceType=mapping", "mapFile="+name,
                 "mapCoeff="+mapCoeff, "mapShift="+mapShift]
        lines[si] = ", ".join(words) + "\n"
        lines[si+1] = sgrpName + ", S, 0.0\n"
        newLines = lines[:si+2] + addLines + lines[ei:]
        f = open(cntFile, "w"); f.writelines(newLines); f.close()

#
#  setFrontIstr_fsi
#---------------------
def setFrontIstr_fsi():
    """ FrontISTRのfsi計算前の設定"""
    #msh,cntファイル名を設定
    setFsiChtCntFile("fsi")
    #時間増分を更新
    endStep, endTime = setDeltaTInCnt_fsi()
    return endStep, endTime

#
#  setFsiChtCntFile
def setFsiChtCntFile(sect):
    """ sectに応じたdatファイルに修正"""
    solidCaseDir = couplingDict["solidCaseDir"]
    pasteDatFile = solidCaseDir + os.sep + "hecmw_ctrl.dat"
    if sect == "fsi":
        datFile = solidCaseDir + os.sep + "hecmw_ctrl_fsi.dat"
    elif sect == "cht":
        datFile = solidCaseDir + os.sep + "hecmw_ctrl_cht.dat"
    elif sect == "chtss":
        datFile = solidCaseDir + os.sep + "hecmw_ctrl_chtss.dat"
    shutil.copy(datFile, pasteDatFile)

#
#  setDeltaTInCnt_fsi
def setDeltaTInCnt_fsi():
    """ fsiのcntファイルを更新する"""
    deltaT = couplingDict["deltaT"]
    solidCaseDir = couplingDict["solidCaseDir"]
    timeIndex = couplingDict["timeIndex"]
    currTime = couplingDict["timeName"]
    startTime = couplingDict["startTime"]
    stepTime = float(deltaT)
    runStep = int(timeIndex)
    #cnt読み込み
    cntFile = solidCaseDir + "/FistrModel_fsi.cnt"
    f = open(cntFile); lines = f.readlines(); f.close()
    #時間増分の設定
    i = pyFistr.getNextHeaderName(lines, 0, "!DYNAMIC", "")
    i += 2
    words = pyFistr.deleteSp(lines[i]).split(",")
    startStep = runStep
    cTime = float(currTime)
    orgStep = 0
    writeInterval = float(couplingDict["writeInterval"])
    nSteps = int(couplingDict["nStepsOF"])
    endStep, endTime = pyCoupling.getEndStepFromCouplingWriteInterval(
        startStep, cTime, stepTime, nSteps, writeInterval, orgStep)
    words[2] = str(endStep)         #endStep
    #時間増分セット
    words[3] = str(stepTime)        #時間増分
    lines[i] = ", ".join(words) + "\n"
    #restartの設定
    if float(currTime) == 0.0:
        #初回は、restartさせずに起動
        i = pyFistr.getNextHeaderName(lines, 0, "!RESTART", "")
        lines[i] = pyFistr.setValName(lines[i], "FREQUENCY=1")
    else:
        #restartで起動する様に修正
        i = pyFistr.getNextHeaderName(lines, 0, "!RESTART", "")
        lines[i] = pyFistr.setValName(lines[i], "FREQUENCY=-1")
    #cntFile保存
    f = open(cntFile, "w"); f.writelines(lines); f.close()
    return endStep, endTime

#
#  runFistr
#-----------
def runFistr(sect, endStep, endTime):
    """ FrontISTRを起動"""
    timeName = couplingDict["timeName"]
    timeIndex = couplingDict["timeIndex"]
    print("(py) --- FrontISTR is running for structure from step "
           + timeIndex + " to " + str(endStep) + " time "
           + str(endTime) + " ...", flush=True)

    #Fistrがfreeまで待つ
    waitUntilFreeFistr()
    #起動設定
    stepCont = sect + " -> " + str(endStep) + " steps"
    title = "title runFistr\n"
    time = "time " + timeName + "\n"
    footer = "footer " + "FrontISTR total time (" + stepCont + ") ----> \n"
    cont = title + time + footer + "#end\n"
    writeCommonArea(cont)
    #Fistr終了まで待つ
    waitUntilFreeFistr()

#
#  waitUntilFreeFistr
def waitUntilFreeFistr():
    """ Fistrの終了を待つ。
    共有メモリの内容から確認する"""
    count = waitCount
    sleepTime = waitTime
    while True:
        cont = readCommonArea()
        if len(cont) > 0:
            lines = cont.split("\n")
            if len(lines) > 0:
                words = lines[0].split()
                if len(words) >= 2:
                    if words[1] == "FistrFinished":
                        break
        else:
            break
        count -= 1
        if count < 0:
            break
        time.sleep(sleepTime)
    return

#
#  createDispPtc
#-----------------
def createDispPtc(endStep):
    """ 変位の点群データを取得"""
    solidCaseDir = couplingDict["solidCaseDir"]
    nodeDictName = solidCaseDir + "/nodeDict.bin"
    f = open(nodeDictName, "rb")
    nodeDict = pickle.load(f)
    f.close()
    fsiDispPtc = createFsiDispPtc(nodeDict, endStep)
    return fsiDispPtc

#
#  createFsiDispPtc
def createFsiDispPtc(nodeDict, endStep):
    """ 変位の点群データ取得"""
    solidCaseDir = couplingDict["solidCaseDir"]
    #計算したstepを取得
    step = endStep
    #resFile名を取得、読み込み
    resFile = solidCaseDir + "/FistrModel.res.0." + str(step)
    waitUntilExistingFile(resFile)
    dispDict = readDispFromResFile(resFile)
    #点群作成
    dispPtc = []
    for nodeNo in nodeDict.keys():
        disp = dispDict[nodeNo]
        loc = nodeDict[nodeNo]
        dispPtc.append([loc, disp])
    return dispPtc

#
#  readDispFromResFile
def readDispFromResFile(resFile):
    """ 変位データをresFileから取得"""
    #resFileを読み込み
    f = open(resFile);
    #*dataまでskip
    while True:
        line = f.readline()
        if line[:len("*data")] == "*data":
            break
    #node数読み込み
    line = f.readline()
    nNodes = int(line.split()[0])
    #節点の値の位置を取得
    _dummy = f.readline()
    line = f.readline()
    words = line.split()
    nums = list(map(int, words))
    nVals = sum(nums)
    dispLoc = 0
    flag = 0
    for i in range(len(words)):
        valName = f.readline().split()[0]
        if valName == "DISPLACEMENT":
            flag = 1
        else:
            if flag == 0:
                dispLoc += nums[i]
    #DISPLACEMENTの値を読み込む
    dispDict = {}
    n = 0
    while True:
        #nodeNo取得
        line = f.readline()
        nodeNo = int(line.split()[0])
        #disp = [nodeNo]
        #disp読み込み
        vals = []
        while True:
            line = f.readline()
            words = line.split()
            vals += words
            if len(vals) >= nVals:
                break
        dispWords = vals[dispLoc:dispLoc+3]
        dispVal = list(map(float, dispWords))
        dispDict[nodeNo] = dispVal
        n += 1
        if n >= nNodes:
            break
    f.close()
    return dispDict

#
#  createMappingDispFile
def createMappingDispFile(dispPtc, endStep):
    """ 変位をpatchにmappingしたfileを作成する"""
    nProcs = couplingDict["nProcs"]
    np = int(nProcs)
    queue = []
    procs = []
    for n in range(np):
        q = multiprocessing.Queue()
        queue.append(q)
        procNo = str(n)
        p = multiprocessing.Process(
            target=createMappingDispFile_proc,
            args=(dispPtc, endStep, procNo, q)
            )
        p.start()
        procs.append(p)
    #全セットprocessの終了を待つ
    for n in range(np):
        a = queue[n].get()

#
#  createMappingDispFile_proc
def createMappingDispFile_proc(dispPtc, endStep, procNo, q):
    """ 指定したprocNoの変位をpatchにmappingしたfileを作成する"""
    solidCaseDir = couplingDict["solidCaseDir"]
    mesh = mapping.fistrMesh(solidCaseDir) 
    #mappingするpointの座標を取得
    mapPointFaceDict, mapPointDict = getMappedPointLocsFsi(procNo)
    #mappingTriFileがあるか？
    if isMappingTriFileFsi(procNo) == False:
        #存在しない場合、mappingTriDictを取得
        mapStat, mappingTriDict = mesh.getMappingTriLocs(dispPtc, mapPointFaceDict, mapPointDict)
        if mapStat != "OK":
            return "error in displacement mapping."
        #mappingTriFileを作成
        createMappingTriFileFsi(mappingTriDict, procNo)
    else:
        #fileを読み込む
        mappingTriDict = readMappingTriFileFsi(procNo)
    #変位をmapping
    mappingDict = mesh.setMappingDataFromTriLocs(dispPtc, mapPointFaceDict, mappingTriDict)
    #mapping結果を保存(このfileをOpenFOAM側が読み込む)
    saveMappingDisp(mappingDict, endStep, procNo)
    q.put("")

#
#  getMappedPointLocsFsi
def getMappedPointLocsFsi(procNo):
    """ 変位をmappingするpointの座標を取得する"""
    patchName = couplingDict["mappingPatch"]
    #procNo = couplingDict["procNo"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    name = patchName + "_mapPointFaceNodes_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    f = open(fileName); lines = f.readlines(); f.close()
    mapPointFaceDict = {}
    mapPointDict = {}
    for line in lines:
        words = line.split()
        if len(words) > 4:
            nodeNo = int(words[0])
            pointLoc = list(map(float, words[1:4]))
            faceNodes = list(map(int, words[4:]))
            mapPointFaceDict[nodeNo] = [pointLoc, faceNodes]
            mapPointDict[nodeNo] = pointLoc
    return mapPointFaceDict, mapPointDict

#
#  isMappingTriFileFsi
def isMappingTriFileFsi(procNo):
    """ 変位mapping用の3点を取得したfileが存在するかチェック"""
    #procNo = couplingDict["procNo"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    name = "mappingTriFile_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    return os.path.exists(fileName)

#
#  createMappingTriFileFsi
def createMappingTriFileFsi(mappingTriDict, procNo):
    """ mapping用の3点の座標をfileに保存する"""
    #procNo = couplingDict["procNo"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    name = "mappingTriFile_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    lines = []
    for key in mappingTriDict.keys():
        faceNo = key
        word = str(faceNo)
        triLocs = mappingTriDict[key]
        loc0, loc1, loc2 = triLocs
        word0 = " ".join(list(map(str, loc0)))
        word1 = " ".join(list(map(str, loc1)))
        word2 = " ".join(list(map(str, loc2)))
        line = " ".join([word, word0, word1, word2]) + "\n"
        lines.append(line)
    f = open(fileName, "w"); f.writelines(lines); f.close()

#
#  readMappingTriFileFsi
def readMappingTriFileFsi(procNo):
    """ 保存したmapping用の3点の座標を読み込む"""
    #procNo = couplingDict["procNo"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    name = "mappingTriFile_" + procNo
    fileName = fluidCaseDir + "/coupling_FrontISTR/data/" + name
    f = open(fileName); lines = f.readlines(); f.close()
    mappingTriDict = {}
    for line in lines:
        words = line.split()
        faceNo = int(words[0])
        key = faceNo
        loc0 = list(map(float, words[1:4]))
        loc1 = list(map(float, words[4:7]))
        loc2 = list(map(float, words[7:10]))
        mappingTriDict[key] = [loc0, loc1, loc2]
    return mappingTriDict

#
#  saveMappingDisp
def saveMappingDisp(mappingDict, endStep, procNo):
    """ mappingDispの結果を保存。このファイルをOF側が読み込む"""
    fluidCaseDir = couplingDict["fluidCaseDir"]
    #procNo = couplingDict["procNo"]
    patchName = couplingDict["mappingPatch"]
    saveDir = fluidCaseDir + "/coupling_FrontISTR/data"
    name = patchName + "_mapPointFaceNodes_" + procNo
    fileName = saveDir + "/" + name
    f = open(fileName); lines = f.readlines(); f.close()
    #保存のfilename
    if procNo == "0":
        print("(py) --- displacement file is creating for OpenFOAM at " + str(endStep) + "...")
    name = patchName + "_disp_" + str(endStep) + "_" + procNo + "tm"
    dispFile = saveDir + "/" + name
    f = open(dispFile, "w")
    for line in lines:
        words = line.split()
        nodeNo = int(words[0])
        disp = mappingDict[nodeNo]
        line = " ".join(list(map(str, disp))) + "\n"
        f.write(line)
    f.close()
    return

#
#  deleteResSetRestartFileFsi
#---------------------------
def deleteResSetRestartFileFsi():
    """ 結果fileを削除する。
    ただし、writeInterval時は、結果fileとrestartFileを残す"""
    timeName = couplingDict["timeName"]
    writeInterval = couplingDict["writeInterval"]
    deltaT = couplingDict["deltaT"]
    timeIndex = couplingDict["timeIndex"]
    solidCaseDir = couplingDict["solidCaseDir"]
    fluidCaseDir = couplingDict["fluidCaseDir"]
    patchName = couplingDict["mappingPatch"]
    nStepsOF = couplingDict["nStepsOF"]
    #writeIntervalかどうか？
    dataDir = fluidCaseDir + "/coupling_FrontISTR/data/"
    #前のcoupling時を取得
    bEndStep, bEndTime = pyCoupling.getBeforeEndStepFromCoupling(
        int(timeIndex), float(timeName), float(deltaT), 
        int(nStepsOF), float(writeInterval))
    #delTime = pyCoupling.time2str(bEndTime)
    delStep = str(bEndStep)
    delTime = pyCoupling.time2str(bEndStep * float(deltaT))
    #delTime = timeName
    #delStep = timeIndex
    delFiles = []
    delFolders = []
    #OpenFOAM側を確認
    if pyCoupling.isWriteInterval(delTime, writeInterval, deltaT) == False:
        #削除可能な時間
        #  圧力の点群ファイルを削除
        pattern = solidCaseDir + "/" + patchName + "_faceCenterPress_" + delTime + "_*.particles"
        fileNames = getAllProcsFiles(pattern, int(nProcs))                
        delFiles += fileNames
        #  dispを削除
        pattern = dataDir + patchName + "_disp_" + delStep + "_*tm"
        fileNames = getAllProcsFiles(pattern, int(nProcs))
        delFiles += fileNames
        #  facePressを削除
        pattern = dataDir + patchName + "_facePress_" + delTime + "_*tm"
        fileNames = getAllProcsFiles(pattern, int(nProcs))
        delFiles += fileNames
    else:
        #writeInterval時の場合
        #  圧力の点群ファイル
        pattern = solidCaseDir + "/" + patchName + "_faceCenterPress_" + delTime + "_*.particles"
        fileNames = getAllProcsFiles(pattern, int(nProcs))                
        delFiles += fileNames
        #  dispをrename
        pattern = dataDir + patchName + "_disp_" + delStep + "_*tm"
        fileNames = getAllProcsFiles(pattern, int(nProcs))
        delete2charaFromEndName(fileNames)
        #  facePressをrename
        pattern = dataDir + patchName + "_facePress_" + delTime + "_*tm"
        fileNames = getAllProcsFiles(pattern, int(nProcs))
        delete2charaFromEndName(fileNames)
    #FrontISTR側を確認
    if pyCoupling.isWriteInterval(delTime, writeInterval, deltaT) == False:
        #削除可能な時間
        #  resFileを削除
        resFile = solidCaseDir + "/FistrModel.res.0." + delStep
        delFiles += [resFile]
        #  vtuFileを削除
        vtuFile = solidCaseDir + "/FistrModel.vis_psf." + delStep.zfill(4) + ".pvtu"
        delFiles += [vtuFile]
        #  vtuFolder
        vtuFolder = solidCaseDir + "/FistrModel.vis_psf." + delStep.zfill(4)
        delFolders += [vtuFolder]
    #FrontISTRのrestartFileを保存
    #calcEnd = float(delTime) + int(nStepsOF) * float(deltaT)
    #calcEndTime = pyCoupling.time2str(calcEnd)
    calcEnd, _time = pyCoupling.getEndStepFromCouplingWriteInterval(
        int(timeIndex), float(timeName), float(deltaT), int(nStepsOF), float(writeInterval), 0)
    calcEndTimeNum = calcEnd * float(deltaT)
    calcEndTime = pyCoupling.time2str(calcEndTimeNum)
    if pyCoupling.isWriteInterval(calcEndTime, writeInterval, deltaT) == True:
        #writeInterval時の時
        #  resFileをrenameして保存
        #calcEndIdx = str(int(delStep) + int(nStepsOF)) 
        calcEndIdx = str(calcEnd)
        saveRestartFileFsi(calcEndTime, calcEndIdx)
    #fileFolderを削除
    for delFile in delFiles:
        if os.path.exists(delFile) == True:
            os.remove(delFile)
    for delFolder in delFolders:
        if os.path.isdir(delFolder) == True:
            shutil.rmtree(delFolder)

#
#  getAllProcsFiles
def getAllProcsFiles(pattern, np):
    """ 全procsのfile名を取得して返す"""
    names = pattern.split("*")
    files = []
    for i in range(np):
        fileName = names[0] + str(i) + names[1]
        files.append(fileName)
    return files

#
#  delete2charaFromEndName
def delete2charaFromEndName(fileNames):
    """ fileNameの最後の2文字(tm)を削除"""
    for fileName in fileNames:
        if os.path.exists(fileName):
            os.rename(fileName, fileName[:-2])

#
#  saveRestartFileFsi
def saveRestartFileFsi(timeName, timeIndex):
    """ FrontISTRのrestartFileを設定する"""
    if int(timeIndex) == 0:
        return
    #restartFile保存
    solidCaseDir = couplingDict["solidCaseDir"]
    orgName = solidCaseDir + "/FistrModel.restart.0"
    newName = solidCaseDir + "/FistrModel.restart_" + timeName + "_" + timeIndex + ".0"
    shutil.copy(orgName, newName)



if __name__ == "__main__":
    import gettext
    gettext.install("app") # replace with the appropriate catalog name
    _ = gettext.gettext

    currTime = sys.argv[1]      #currTime
    beforeTime = sys.argv[2]    #前回のtime
    procNo = sys.argv[3]        #procNo
    nProcs = sys.argv[4]        #nProcs
    timeIndex = sys.argv[5]     #timeIndex
    fluidCaseDir = os.getcwd()
    couplingData = pyCoupling.couplingData(fluidCaseDir)
    couplingDict = couplingData.read()
    couplingDict["fluidCaseDir"] = fluidCaseDir
    couplingDict["timeName"] = currTime
    couplingDict["beforeTime"] = beforeTime
    couplingDict["procNo"] = procNo
    couplingDict["nProcs"] = nProcs
    couplingDict["timeIndex"] = timeIndex
    waitCount = int(couplingDict["maxWaitTime"])
    #共有メモリの設定
    mmCommon = createCommonArea()
    #現在時間が終了時間？
    if float(couplingDict["timeName"]) >= float(couplingDict["endTime"]):
        exit()
    #mainへ
    stat = coupling_fsi()
    if stat != "":
        raise NameError(stat)
